
MTC_DataID_list = ['ACTUATOR','CHUCK_INTERLOCK','COMMUNICATIONS','DATA_RANGE','DIRECTION','END_OF_BAR','HARDWARE','INTERFACE_STATE','LOGIC_PROGRAM','MOTION_PROGRAM','SYSTEM','ACCELERATION','ACCUMULATED_TIME','ANGULAR_ACCELERATION','ANGULAR_VELOCITY','AMPERAGE','ALTERNATING','DIRECT','ACTUAL','TARGET','ANGLE','ACTUAL','COMMANDED','AXIS_FEEDRATE','ACTUAL','COMMANDED','JOG','PROGRAMMED','RAPID','OVERRIDE','CLOCK_TIME','CONCENTRATION','CONDUCTIVITY','DISPLACEMENT','ELECTRICAL_ENERGY','EQUIPMENT_TIMER','LOADED','WORKING','OPERATING','POWERED','DELAY','FILL_LEVEL','FLOW','FREQUENCY','LENGTH','STANDARD','REMAINING','USEABLE','LINEAR_FORCE','LOAD','MASS','PATH_FEEDRATE','ACTUAL','COMMANDED','JOG','PROGRAMMED','RAPID','OVERRIDE','PATH_POSITION','ACTUAL','COMMANDED','TARGET','PROBE','PH','POSITION','ACTUAL','COMMANDED','PROGRAMMED','TARGET','POWER_FACTOR','PRESSURE','PROCESS_TIMER','PROCESS','DELAY','RESISTANCE','ROTARY_VELOCITY','ACTUAL','COMMANDED','PROGRAMMED','OVERRIDE','SOUND_LEVEL','NO_SCALE','A_SCALE','B_SCALE','C_SCALE','D_SCALE','SPINDLE_SPEED','ACTUAL','COMMANDED','OVERRIDE','STRAIN','TEMPERATURE','TENSION','TILT','TORQUE','VOLT_AMPERE','VOLT_AMPERE_REACTIVE','VELOCITY','VISCOSITY','VOLTAGE','ALTERNATING','DIRECT','ACTUAL','TARGET','WATTAGE','ACTUAL','TARGET','ACTUATOR_STATE','ALARM','ACTIVE_AXES','','','AVAILABILITY','AXIS_COUPLING','','','AXIS_FEEDRATE_OVERRIDE','JOG','PROGRAMMED','RAPID','AXIS_INTERLOCK','AXIS_STATE','BLOCK','BLOCK_COUNT','CHUCK_INTERLOCK','MANUAL_UNCLAMP','CHUCK_STATE','CODE','COMPOSITION_STATE','ACTION','LATERAL','MOTION','SWITCHED','VERTICAL','CONTROLLER_MODE','CONTROLLER_MODE_OVERRIDE','DRY_RUN','SINGLE_BLOCK','MACHINE_AXIS_LOCK','OPTIONAL_STOP','TOOL_CHANGE_STOP','COUPLED_AXES','DIRECTION','ROTARY','LINEAR','DOOR_STATE','END_OF_BAR','PRIMARY','AUXILIARY','EMERGENCY_STOP','EQUIPMENT_MODE','LOADED','WORKING','OPERATING','POWERED','DELAY','EXECUTION','FUNCTIONAL_MODE','HARDNESS','ROCKWELL','VICKERS','SHORE','BRINELL','LEEB','MOHS','INTERFACE_STATE','LINE','MAXIMUM','MINIMUM','LINE_LABEL','LINE_NUMBER','ABSOLUTE','INCREMENTAL','MATERIAL','MESSAGE','OPERATOR_ID','PALLET_ID','PART_COUNT','ALL','GOOD','BAD','TARGET','REMAINING','PART_ID','PART_NUMBER','PATH_FEEDRATE_OVERRIDE','JOG','PROGRAMMED','RAPID','PATH_MODE','POWER_STATE','','','LINE','CONTROL','POWER_STATUS','PROGRAM','PROGRAM_EDIT','PROGRAM_EDIT_NAME','PROGRAM_COMMENT','PROGRAM_HEADER','ROTARY_MODE','ROTARY_VELOCITY_OVERRIDE','SERIAL_NUMBER','SPINDLE_INTERLOCK','TOOL_ASSET_ID','TOOL_NUMBER','TOOL_OFFSET','RADIAL','LENGTH','USER','OPERATOR','MAINTENANCE','SET_UP','WIRE','WORKHOLDING_ID','WORK_OFFSET']